@@ -159,15 +159,13 @@ static void gtk_gs_init (GtkGS *gs);
 static void gtk_gs_class_init (GtkGSClass *klass);
 static void gtk_gs_destroy (GtkObject *object);
 static void gtk_gs_realize (GtkWidget *widget);
-static void gtk_gs_unrealize (GtkWidget *widget);
 static void gtk_gs_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void gtk_gs_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint gtk_gs_widget_event(GtkWidget *widget, GdkEvent *event, gpointer data);
 static void gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data);
 static void gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data);
 static void gtk_gs_emit_error_msg (GtkGS *gs, const gchar *msg);
-static void gtk_gs_defaults_changed (GConfClient* client, guint cnxn_id,
-                                     GConfEntry *entry, gpointer user_data);
+static void gtk_gs_set_adjustments (GtkGS *gs, GtkAdjustment *hadj, GtkAdjustment *vadj);
 static void send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close);
 static void set_up_page (GtkGS *gs);
 static void close_pipe (int p[2]);
@@ -191,66 +189,100 @@ static gint gtk_gs_signals[LAST_SIGNAL] = { 0 };
 
 static void
 ggv_marshaller_VOID__POINTER(GClosure     *closure,
-				  GValue       *return_value,
-                                  guint         n_param_values,
-                                  const GValue *param_values,
-                                  gpointer      invocation_hint,
-                                  gpointer      marshal_data)
+                             GValue       *return_value,
+                             guint         n_param_values,
+                             const GValue *param_values,
+                             gpointer      invocation_hint,
+                             gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER) (gpointer     data1,
-                                              gpointer     arg_1,
-                                              gpointer     data2);
-  register GMarshalFunc_VOID__POINTER callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
+        typedef void (*GMarshalFunc_VOID__POINTER) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     data2);
+        register GMarshalFunc_VOID__POINTER callback;
+        register GCClosure *cc = (GCClosure*) closure;
+        register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 2);
+        g_return_if_fail (n_param_values == 2);
 
-  if (G_CCLOSURE_SWAP_DATA (closure)) {
-          data1 = closure->data;
-          data2 = g_value_peek_pointer (param_values + 0);
-  }
-  else {
-          data1 = g_value_peek_pointer (param_values + 0);
-          data2 = closure->data;
-  }
-  callback = (GMarshalFunc_VOID__POINTER) (marshal_data ? marshal_data : cc->callback);
-
-  callback (data1,
-            g_value_get_pointer (param_values + 1),
-            data2);
+        if (G_CCLOSURE_SWAP_DATA (closure)) {
+                data1 = closure->data;
+                data2 = g_value_peek_pointer (param_values + 0);
+        }
+        else {
+                data1 = g_value_peek_pointer (param_values + 0);
+                data2 = closure->data;
+        }
+        callback = (GMarshalFunc_VOID__POINTER) (marshal_data ? marshal_data : cc->callback);
+        
+        callback (data1,
+                  g_value_get_pointer (param_values + 1),
+                  data2);
 }
 
 static void
 ggv_marshaller_VOID__INT(GClosure     *closure,
-				  GValue       *return_value,
-                                  guint         n_param_values,
-                                  const GValue *param_values,
-                                  gpointer      invocation_hint,
-                                  gpointer      marshal_data)
+                         GValue       *return_value,
+                         guint         n_param_values,
+                         const GValue *param_values,
+                         gpointer      invocation_hint,
+                         gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__INT) (gpointer     data1,
-                                          gint         arg_1,
-                                          gpointer     data2);
-  register GMarshalFunc_VOID__INT callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
+        typedef void (*GMarshalFunc_VOID__INT) (gpointer     data1,
+                                                gint         arg_1,
+                                                gpointer     data2);
+        register GMarshalFunc_VOID__INT callback;
+        register GCClosure *cc = (GCClosure*) closure;
+        register gpointer data1, data2;
+        
+        g_return_if_fail (n_param_values == 2);
+        
+        if (G_CCLOSURE_SWAP_DATA (closure)) {
+                data1 = closure->data;
+                data2 = g_value_peek_pointer (param_values + 0);
+        }
+        else {
+                data1 = g_value_peek_pointer (param_values + 0);
+                data2 = closure->data;
+        }
+        callback = (GMarshalFunc_VOID__INT) (marshal_data ? marshal_data : cc->callback);
+        
+        callback (data1,
+                  g_value_get_int (param_values + 1),
+                  data2);
+}
 
-  g_return_if_fail (n_param_values == 2);
+void
+ggv_marshaller_VOID__POINTER_POINTER (GClosure     *closure,
+                                      GValue       *return_value,
+                                      guint         n_param_values,
+                                      const GValue *param_values,
+                                      gpointer      invocation_hint,
+                                      gpointer      marshal_data)
+{
+        typedef void (*GMarshalFunc_VOID__POINTER_POINTER) (gpointer     data1,
+                                                            gpointer     arg_1,
+                                                            gpointer     arg_2,
+                                                            gpointer     data2);
+        register GMarshalFunc_VOID__POINTER_POINTER callback;
+        register GCClosure *cc = (GCClosure*) closure;
+        register gpointer data1, data2;
+        
+        g_return_if_fail (n_param_values == 3);
+        
+        if (G_CCLOSURE_SWAP_DATA (closure)) {
+                data1 = closure->data;
+                data2 = g_value_peek_pointer (param_values + 0);
+        }
+        else {
+                data1 = g_value_peek_pointer (param_values + 0);
+                data2 = closure->data;
+        }
+        callback = (GMarshalFunc_VOID__POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);
 
-  if (G_CCLOSURE_SWAP_DATA (closure)) {
-          data1 = closure->data;
-          data2 = g_value_peek_pointer (param_values + 0);
-  }
-  else {
-          data1 = g_value_peek_pointer (param_values + 0);
-          data2 = closure->data;
-  }
-  callback = (GMarshalFunc_VOID__INT) (marshal_data ? marshal_data : cc->callback);
-
-  callback (data1,
-            g_value_get_int (param_values + 1),
-            data2);
+        callback (data1,
+                  g_value_get_pointer (param_values + 1),
+                  g_value_get_pointer (param_values + 2),
+                  data2);
 }
 
 static void
@@ -318,11 +350,6 @@ gtk_gs_init (GtkGS *gs)
         gs->show_scroll_rect = gtk_gs_defaults_get_show_scroll_rect();
 
         gs->scroll_start_x = gs->scroll_start_y = -1;
-        
-        gs->gconf_notify_id = gconf_client_notify_add
-                (gtk_gs_defaults_gconf_client(), "/apps/ggv/gtkgs",
-                 (GConfClientNotifyFunc)gtk_gs_defaults_changed,
-                 gs, NULL, NULL);
 }
 
 static void
@@ -375,6 +402,17 @@ gtk_gs_class_init (GtkGSClass *klass)
         widget_class->realize       = gtk_gs_realize;
         widget_class->size_request  = gtk_gs_size_request;
         widget_class->size_allocate = gtk_gs_size_allocate;
+        widget_class->set_scroll_adjustments_signal =
+                g_signal_new ("set_scroll_adjustments",
+                              G_TYPE_FROM_CLASS(object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET(GtkGSClass, set_scroll_adjustments),
+                              NULL,
+                              NULL,
+                              ggv_marshaller_VOID__POINTER_POINTER,
+                              G_TYPE_NONE, 2,
+                              G_TYPE_POINTER,
+                              G_TYPE_POINTER);
 
         /* Create atoms */
         klass->gs_atom        = gdk_atom_intern ("GHOSTVIEW", FALSE);
@@ -386,43 +424,12 @@ gtk_gs_class_init (GtkGSClass *klass)
 
         /* a default handler for "interpreter_message" signal */
         klass->interpreter_message = gtk_gs_interpreter_message;
+        /* supply a scrollable interface */
+        klass->set_scroll_adjustments = gtk_gs_set_adjustments;
+
         gtk_gs_defaults_load();
 }
 
-static void
-gtk_gs_defaults_changed(GConfClient* client, guint cnxn_id,
-                        GConfEntry *entry, gpointer user_data)
-{
-        GtkGS *gs = GTK_GS(user_data);
-
-        if(!strcmp(entry->key, "/apps/ggv/gtkgs/respect_eof"))
-                gtk_gs_set_respect_eof(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/respect_eof", NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/override_orientation"))
-                gtk_gs_set_override_orientation(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/override_orientation",
-                                                                          NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/orientation"))
-                gtk_gs_set_default_orientation(gs, gconf_client_get_int(client, "/apps/ggv/gtkgs/orientation",
-                                                                        NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/zoom"))
-                gtk_gs_set_zoom(gs, gconf_client_get_float(client, "/apps/ggv/gtkgs/zoom",
-                                                           NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/size"))
-                gtk_gs_set_default_size(gs, gconf_client_get_int(client, "/apps/ggv/gtkgs/size",
-                                                                  NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/antialiasing"))
-                gtk_gs_set_antialiasing(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/antialiasing",
-                                                                  NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/override_size"))
-                gtk_gs_set_override_size(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/override_size",
-                                                                    NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/watch_doc"))
-                gtk_gs_set_watch_file(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/watch_doc",
-                                                                NULL));
-        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/show_scroll_rect"))
-                gtk_gs_set_show_scroll_rect(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/show_scroll_rect",
-                                                                      NULL));
-}
-
 /* Clean all memory and temporal files */
 static void 
 gtk_gs_cleanup (GtkGS *gs)
@@ -503,6 +510,20 @@ gtk_gs_destroy (GtkObject *object)
                                            gs->gconf_notify_id);
                 gs->gconf_notify_id = -1;
         }
+        if(gs->hadj) {
+                g_signal_handlers_disconnect_matched (G_OBJECT (gs->hadj),
+                                                      G_SIGNAL_MATCH_DATA,
+                                                      0, 0, NULL, NULL, gs);
+                gtk_object_unref (GTK_OBJECT (gs->hadj));
+                gs->hadj = NULL;
+        }
+        if(gs->vadj) {
+                g_signal_handlers_disconnect_matched (G_OBJECT (gs->vadj),
+                                                      G_SIGNAL_MATCH_DATA,
+                                                      0, 0, NULL, NULL, gs);
+                gtk_object_unref (GTK_OBJECT (gs->vadj));
+                gs->vadj = NULL;
+        }
 
         if (GTK_OBJECT_CLASS (parent_class)->destroy)
                 (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -534,6 +555,7 @@ gtk_gs_munge_adjustments (GtkGS *gs)
 		else if (gs->widget.allocation.width > x + gs->width) 
 			x = gs->widget.allocation.width - gs->width;
 		gs->hadj->page_size = ((gfloat) gs->widget.allocation.width)/gs->width;
+                gs->hadj->page_increment = gs->hadj->page_size*0.9;
 		gs->hadj->value = -((gfloat)x)/gs->width;
 	}
 	if (gs->widget.allocation.height >= gs->height) {
@@ -547,6 +569,7 @@ gtk_gs_munge_adjustments (GtkGS *gs)
 		else if (gs->widget.allocation.height > y + gs->height) 
 			y = gs->widget.allocation.height - gs->height;
 		gs->vadj->page_size = ((gfloat) gs->widget.allocation.height)/gs->height;
+                gs->vadj->page_increment = gs->vadj->page_size*0.9;
 		gs->vadj->value = -((gfloat)y)/gs->height;
 	}
 
@@ -618,35 +641,12 @@ gtk_gs_realize (GtkWidget *widget)
 		gtk_gs_munge_adjustments (gs);
         }
 
-        g_signal_connect(GTK_OBJECT(widget), "event",
+        g_signal_connect(G_OBJECT(widget), "event",
                          G_CALLBACK(gtk_gs_widget_event), gs);
 
         gtk_gs_goto_page(gs, gs->current_page);
 }
 
-static void
-gtk_gs_unrealize (GtkWidget *widget)
-{
-        GtkGS  *gs;
-
-        g_return_if_fail (widget != NULL);
-        g_return_if_fail (GTK_IS_GS (widget));
-
-        gs = GTK_GS (widget);
-
-        if(gs->pstarget) {
-                gdk_window_destroy(gs->pstarget);
-                gs->pstarget = NULL;
-        }
-        if(gs->psgc) {
-                gdk_gc_destroy(gs->psgc);
-                gs->psgc = NULL;
-        }
-
-        if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-                (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
-}
-
 static void
 gtk_gs_size_request (GtkWidget      *widget,
                      GtkRequisition *requisition)
@@ -873,30 +873,34 @@ set_up_page (GtkGS *gs)
 
                 /* clear new pixmap (set to white) */
                 fill = gdk_gc_new (gs->pstarget);
-                colormap = gtk_widget_get_colormap (GTK_WIDGET(gs));
-                gdk_color_alloc (colormap, &white);
-                gdk_gc_set_foreground (fill, &white);
+                if(fill) {
+                        colormap = gtk_widget_get_colormap (GTK_WIDGET(gs));
+                        gdk_color_alloc (colormap, &white);
+                        gdk_gc_set_foreground (fill, &white);
  
-                if(gs->use_bpixmap) {
-                        if (gs->bpixmap) {
-                                gdk_drawable_unref(gs->bpixmap);
-                                gs->bpixmap = NULL;
+                        if(gs->use_bpixmap && gs->width > 0 && gs->height > 0) {
+                                if (gs->bpixmap) {
+                                        gdk_drawable_unref(gs->bpixmap);
+                                        gs->bpixmap = NULL;
+                                }
+                        
+                                gs->bpixmap = gdk_pixmap_new(gs->pstarget, gs->width, gs->height, -1);
+                                
+                                gdk_draw_rectangle (gs->bpixmap, fill, TRUE,
+                                                    0, 0, gs->width, gs->height);
+                                
+                                gdk_window_set_back_pixmap(gs->pstarget, gs->bpixmap, FALSE);
                         }
-                        gs->bpixmap = gdk_pixmap_new(gs->pstarget, gs->width, gs->height, -1);
- 
-                        gdk_draw_rectangle (gs->bpixmap, fill, TRUE,
-                                            0, 0, gs->width, gs->height);
-                        gdk_window_set_back_pixmap(gs->pstarget, gs->bpixmap, FALSE);
-                }
-                else {
-                        gdk_draw_rectangle (gs->pstarget, fill, TRUE,
-                                            0, 0, gs->width, gs->height);
-                }
-                gdk_gc_unref(fill);
+                        else {
+                                gdk_draw_rectangle (gs->pstarget, fill, TRUE,
+                                                    0, 0, gs->width, gs->height);
+                        }
+                        gdk_gc_unref(fill);
 
-                gdk_window_resize (gs->pstarget, gs->width, gs->height); 
+                        gdk_window_resize (gs->pstarget, gs->width, gs->height); 
 
-                gdk_flush();
+                        gdk_flush();
+                }
         }
 
 #ifdef HAVE_SETLOCALE
@@ -1593,26 +1597,8 @@ gtk_gs_new (GtkAdjustment *hadj, GtkAdjustment *vadj)
         g_return_val_if_fail(vadj != NULL, NULL);
 
 	gs = gtk_type_new(gtk_gs_get_type());
-        gs->hadj = hadj;
-        gs->vadj = vadj;
-
-        /* we set the adjustment widget to a default size... */
-        hadj->lower = 0.0;
-        hadj->upper = 1.0;
-        hadj->value = 0.0;
-        hadj->page_size = 1.0;
-        vadj->lower = 0.0;
-        vadj->upper = 1.0;
-        vadj->value = 0.0;
-        vadj->page_size = 1.0;
-
-        g_signal_connect (GTK_OBJECT (hadj), "value_changed",
-                            (GtkSignalFunc) gtk_gs_value_adjustment_changed,
-                            (gpointer) gs);
-        g_signal_connect (GTK_OBJECT (vadj), "value_changed",
-                            (GtkSignalFunc) gtk_gs_value_adjustment_changed,
-                            (gpointer) gs);
 
+        gtk_gs_set_adjustments(gs, hadj, vadj);
 
 	return GTK_WIDGET(gs);
 }
@@ -1626,6 +1612,23 @@ gtk_gs_new_from_file(GtkAdjustment *hadj, GtkAdjustment *vadj, char *fname)
 	return gs;
 }
 
+void
+gtk_gs_reload(GtkGS *gs)
+{
+	gchar *fname;
+	gfloat hval = gs->hadj->value;
+	gfloat vval = gs->vadj->value;
+
+        if(!gs->gs_filename)
+                return;
+
+	fname = g_strdup(gs->gs_filename);
+	gtk_gs_load(gs, fname);
+	gtk_adjustment_set_value(gs->hadj, hval);
+	gtk_adjustment_set_value(gs->vadj, vval);
+	g_free(fname);				
+}
+
 
 /*
  * Show error message -> send signal "interpreter_message"
@@ -1795,22 +1798,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 
                 } else {
                         /* Otherwise, set the proper orientation for the doc */
-                        switch  (gs->doc->orientation) {
-                        case LANDSCAPE:
-                                gs->real_orientation = GTK_GS_ORIENTATION_LANDSCAPE;
-                                break;
-                        case PORTRAIT:
-                                gs->real_orientation = GTK_GS_ORIENTATION_PORTRAIT;
-                                break;
-                        case NONE:
-                                gs->real_orientation = GTK_GS_ORIENTATION_NONE;
-                                break;
-                        default:
-                                gs->real_orientation = GTK_GS_ORIENTATION_NONE;
-                                g_warning("Unknown orientation %d",
-                                          gs->doc->orientation);
-                                break;
-                        }
+                        gs->real_orientation = gs->doc->orientation;
                 }
                 stat(fname, &sbuf);
                 gs->mtime = sbuf.st_mtime;
@@ -1854,9 +1842,10 @@ gtk_gs_next_page(GtkGS *gs)
 	event.xclient.message_type  = gdk_x11_atom_to_xatom(gs_class->next_atom);
 	event.xclient.format        = 32;
         
-        gdk_send_xevent(gs->message_window, FALSE, 0, &event);     
- 
-        gdk_flush();
+        gdk_error_trap_push ();
+        XSendEvent (gdk_display, gs->message_window, FALSE, 0, &event);
+        gdk_flush ();
+        gdk_error_trap_pop ();
 
 	return TRUE;
 }
@@ -1876,10 +1865,14 @@ gtk_gs_get_page_count(GtkGS *gs)
         if(!gs->gs_filename)
                 return 0;
 
-        if(gs->structured_doc && gs->doc)
-                return gs->doc->numpages;
+        if(gs->doc) {
+                if(gs->structured_doc)
+                        return gs->doc->numpages;
+                else
+                        return G_MAXINT;
+        }
         else
-                return G_MAXINT;
+                return 0;
 }
 
 gboolean
@@ -1918,8 +1911,16 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 if(!GTK_WIDGET_REALIZED(gs))
                         return FALSE;
 
+                if(gs->doc->pages[page].orientation != NONE &&
+                   !gs->override_orientation &&
+                   gs->doc->pages[page].orientation != gs->real_orientation) {
+                        gs->real_orientation = gs->doc->pages[page].orientation;
+                        gs->changed = TRUE;
+                        gtk_widget_queue_resize(GTK_WIDGET(gs));
+                }
+
                 gs->changed = FALSE;
-                
+
                 if (is_interpreter_ready (gs)) {
                         gtk_gs_next_page (gs);
                 } else {
@@ -1976,7 +1977,7 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
         if(new_pagesize == -1) {
-                if(gs->default_size > 0)
+                if(gs->default_size >= 0)
                         new_pagesize = gs->default_size;
                 if (!gs->override_size && gs->doc) {
                         /* If we have a document:
@@ -2015,7 +2016,8 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 new_ury = papersizes[new_pagesize].height;
         }
         else if ((gs->doc->epsf && !gs->override_size) || new_pagesize == -1) { /* epsf or bbox */
-                if((gs->doc->pages) &&
+                if((pageid >= 0) &&
+                   (gs->doc->pages) &&
                    (gs->doc->pages[pageid].boundingbox[URX] > gs->doc->pages[pageid].boundingbox[LLX]) &&
                    (gs->doc->pages[pageid].boundingbox[URY] > gs->doc->pages[pageid].boundingbox[LLY])) {
                         /* use page bbox */
@@ -2211,11 +2213,16 @@ gtk_gs_get_orientation(GtkGS *gs)
 {
 	g_return_val_if_fail(gs != NULL, -1);
 	g_return_val_if_fail(GTK_IS_GS(gs), -1);
-        
+
+        if(gs->doc && gs->structured_doc) {
+                if(gs->doc->pages[MAX(gs->current_page, 0)].orientation != GTK_GS_ORIENTATION_NONE)
+                        gs->real_orientation = gs->doc->pages[MAX(gs->current_page, 0)].orientation;
+        }
+
         if (gs->override_orientation ||
             gs->real_orientation == GTK_GS_ORIENTATION_NONE)
                 return gs->fallback_orientation;
-        else
+        else 
                 return gs->real_orientation;
 }
 
@@ -2367,7 +2374,6 @@ void
 gtk_gs_start_scroll(GtkGS *gs)
 {
         gint x, y, w, h;
-        GdkRectangle *rect; 
 
         if(!GTK_WIDGET_REALIZED(gs) || !gs->show_scroll_rect)
                 return;
@@ -2554,16 +2560,35 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         gchar *doc;
         gboolean free_pages = FALSE;
 
-        sink = gtk_gs_doc_sink_new();
-
         if(pages == NULL) {
-                int i, n = gtk_gs_get_page_count(gs);
-                pages = g_new0(gint, n);
-                for(i = 0; i < n; i++)
-                        pages[i] = TRUE;
-                free_pages = TRUE;
+                if(gs->doc->epsf) {
+                        FILE *f;
+                        struct stat sb;
+
+                        if(stat(GTK_GS_GET_PS_FILE(gs), &sb))
+                                return NULL;
+                        doc = g_new(gchar, sb.st_size);
+                        if(!doc)
+                                return NULL;
+                        f = fopen(GTK_GS_GET_PS_FILE(gs), "r");
+                        if(fread(doc, sb.st_size, 1, f) != 1) {
+                                g_free(doc);
+                                doc = NULL;
+                        }
+                        fclose(f);
+                        return doc;
+                }
+                else {
+                        int i, n = gtk_gs_get_page_count(gs);
+                        pages = g_new0(gint, n);
+                        for(i = 0; i < n; i++)
+                                pages[i] = TRUE;
+                        free_pages = TRUE;
+                }
         }
 
+        sink = gtk_gs_doc_sink_new();
+
         if(GTK_GS_IS_PDF(gs)) {
                 /* Save marked pages of the dsc file. The saved
                  * file is useless without the pdf file, but can
@@ -2586,3 +2611,62 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         gtk_gs_doc_sink_free(sink);
         return doc;
 }
+
+void
+gtk_gs_set_adjustments (GtkGS         *gs,
+                        GtkAdjustment *hadj,
+                        GtkAdjustment *vadj)
+{
+        g_return_if_fail (gs != NULL);
+        g_return_if_fail (GTK_IS_GS (gs));
+        if (hadj)
+                g_return_if_fail (GTK_IS_ADJUSTMENT (hadj));
+        else
+                hadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 0.0, 0.0, 1.0));
+        if (vadj)
+                g_return_if_fail (GTK_IS_ADJUSTMENT (vadj));
+        else
+                vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 0.0, 0.0, 1.0));
+  
+        if (gs->hadj && (gs->hadj != hadj)) {
+                g_signal_handlers_disconnect_matched (G_OBJECT (gs->hadj),
+                                                      G_SIGNAL_MATCH_DATA,
+                                                      0, 0, NULL, NULL, gs);
+                gtk_object_unref (GTK_OBJECT (gs->hadj));
+        }
+        if (gs->vadj && (gs->vadj != vadj)) {
+                g_signal_handlers_disconnect_matched (G_OBJECT (gs->vadj),
+                                                      G_SIGNAL_MATCH_DATA,
+                                                      0, 0, NULL, NULL, gs);
+                gtk_object_unref (GTK_OBJECT (gs->vadj));
+        }
+        if (gs->hadj != hadj) {
+                hadj->lower = 0.0;
+                hadj->upper = 1.0;
+                hadj->value = 0.0;
+                hadj->page_size = 1.0;
+                gs->hadj = hadj;
+                gtk_object_ref (GTK_OBJECT (gs->hadj));
+                gtk_object_sink (GTK_OBJECT (gs->hadj));
+                
+                g_signal_connect (G_OBJECT (hadj), "value_changed",
+                                  G_CALLBACK(gtk_gs_value_adjustment_changed),
+                                  (gpointer) gs);       
+        }
+        if (gs->vadj != vadj) {
+                vadj->lower = 0.0;
+                vadj->upper = 1.0;
+                vadj->value = 0.0;
+                vadj->page_size = 1.0;
+                gs->vadj = vadj;
+                gtk_object_ref (GTK_OBJECT (gs->vadj));
+                gtk_object_sink (GTK_OBJECT (gs->vadj));
+
+                g_signal_connect (G_OBJECT (vadj), "value_changed",
+                                  G_CALLBACK(gtk_gs_value_adjustment_changed),
+                                  (gpointer) gs);
+        }
+        if(GTK_WIDGET_REALIZED(gs))
+                gtk_gs_munge_adjustments(gs);
+}
+
