@@ -42,7 +42,7 @@
  *		
  *			<plass@thep.physik.uni-mainz.de>
  */
- 
+
 /* end of patch */
 
 #include <stdlib.h>
@@ -344,12 +344,16 @@ struct document *psscan (FILE *file, int respect_eof)
 	    if (strcmp(text, "(atend)") == 0) {
 		orientation_set = ATEND;
 	    } else if (strcmp(text, "Portrait") == 0) {
-		doc->orientation = PORTRAIT;
+		doc->orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		orientation_set = 1;
 	    } else if (strcmp(text, "Landscape") == 0) {
-		doc->orientation = LANDSCAPE;
+		doc->orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		orientation_set = 1;
 	    }
+            else if(strcmp(text, "Seascape") == 0) {
+                    doc->orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                    orientation_set = 1;
+            }
 	} else if (page_order_set == NONE && iscomment(line+2, "PageOrder:")) {
 	    sscanf(line+length("%%PageOrder:"), "%s", text);
 	    if (strcmp(text, "(atend)") == 0) {
@@ -566,10 +570,13 @@ struct document *psscan (FILE *file, int respect_eof)
 		iscomment(line+2, "PageOrientation:")) {
 		sscanf(line+length("%%PageOrientation:"), "%s", text);
 		if (strcmp(text, "Portrait") == 0) {
-		    doc->default_page_orientation = PORTRAIT;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
-		    doc->default_page_orientation = LANDSCAPE;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		}
+                else if(strcmp(text, "Seascape") == 0) {
+                        doc->default_page_orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                }
 	    } else if (page_size_set == NONE &&
 		       iscomment(line+2, "PageMedia:")) {
 		cp = get_next_text(line+length("%%PageMedia:"), NULL);
@@ -690,10 +697,13 @@ struct document *psscan (FILE *file, int respect_eof)
 		iscomment(line+2, "PageOrientation:")) {
 		sscanf(line+length("%%PageOrientation:"), "%s", text);
 		if (strcmp(text, "Portrait") == 0) {
-		    doc->default_page_orientation = PORTRAIT;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
-		    doc->default_page_orientation = LANDSCAPE;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		}
+                else if(strcmp(text, "Seascape") == 0) {
+                        doc->default_page_orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                }
 	    } else if (page_size_set == NONE &&
 		       iscomment(line+2, "PaperSize:")) {
 		cp = get_next_text(line+length("%%PaperSize:"), NULL);
@@ -829,10 +839,13 @@ continuepage:
 		iscomment(line+2, "PageOrientation:")) {
 		sscanf(line+length("%%PageOrientation:"), "%s", text);
 		if (strcmp(text, "Portrait") == 0) {
-		    doc->pages[doc->numpages].orientation = PORTRAIT;
+		    doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
-		    doc->pages[doc->numpages].orientation = LANDSCAPE;
+		    doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		}
+                else if(strcmp(text, "Seascape") == 0) {
+                        doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                }
 	    } else if (doc->pages[doc->numpages].size == NULL &&
 		       iscomment(line+2, "PageMedia:")) {
 		cp = get_next_text(line+length("%%PageMedia:"), NULL);
@@ -975,10 +988,13 @@ continuepage:
 		   iscomment(line+2, "Orientation:")) {
 	    sscanf(line+length("%%Orientation:"), "%s", text);
 	    if (strcmp(text, "Portrait") == 0) {
-		doc->orientation = PORTRAIT;
+		doc->orientation = GTK_GS_ORIENTATION_PORTRAIT;
 	    } else if (strcmp(text, "Landscape") == 0) {
-		doc->orientation = LANDSCAPE;
+		doc->orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 	    }
+            else if(strcmp(text, "Seascape") == 0) {
+                    doc->orientation = GTK_GS_ORIENTATION_SEASCAPE;
+            }
 	} else if (page_order_set == ATEND && iscomment(line+2, "PageOrder:")) {
 	    sscanf(line+length("%%PageOrder:"), "%s", text);
 	    if (strcmp(text, "Ascend") == 0) {
